@@ -9,7 +9,7 @@
 #
 package MooseX::AttributeShortcuts;
 {
-  $MooseX::AttributeShortcuts::VERSION = '0.016'; # TRIAL
+  $MooseX::AttributeShortcuts::VERSION = '0.018'; # TRIAL
 }
 
 # ABSTRACT: Shorthand for common attribute options
@@ -22,6 +22,7 @@ use namespace::autoclean;
 use Moose ();
 use Moose::Exporter;
 use Moose::Util::MetaRole;
+use Moose::Util::TypeConstraints;
 
 # debug...
 #use Smart::Comments;
@@ -29,14 +30,16 @@ use Moose::Util::MetaRole;
 {
     package MooseX::AttributeShortcuts::Trait::Attribute;
 {
-  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.016'; # TRIAL
+  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.018'; # TRIAL
 }
     use namespace::autoclean;
     use MooseX::Role::Parameterized;
-
+    use Moose::Util::TypeConstraints qw{ class_type role_type enum };
     use MooseX::Types::Moose          ':all';
     use MooseX::Types::Common::String ':all';
 
+    sub _find_or_create_isa_type_constraint { goto \&Moose::Util::TypeConstraints::find_or_create_isa_type_constraint }
+
     parameter writer_prefix  => (isa => NonEmptySimpleStr, default => '_set_');
     parameter builder_prefix => (isa => NonEmptySimpleStr, default => '_build_');
 
@@ -67,11 +70,27 @@ use Moose::Util::MetaRole;
             init_arg  => '_anon_builder',
         );
 
+        has constraint => (
+            is        => 'ro',
+            isa       => 'CodeRef',
+            predicate => 'has_constraint',
+        );
+
+        has original_isa => (
+            is        => 'ro',
+            predicate => 'has_original_isa',
+        );
+
+        # TODO coerce via, transform ?
+
+        # has original_isa, original_coerce ?
+
         my $_process_options = sub {
             my ($class, $name, $options) = @_;
 
-            my $_has = sub { defined $options->{$_[0]} };
+            my $_has = sub { defined $options->{$_[0]}             };
             my $_opt = sub { $_has->(@_) ? $options->{$_[0]} : q{} };
+            my $_ref = sub { ref $_opt->(@_) || q{}                };
 
             if ($options->{is}) {
 
@@ -90,6 +109,54 @@ use Moose::Util::MetaRole;
                 }
             }
 
+            # TODO isa_class - anon class_type generation
+            # TODO isa_role  - anon role_type generation
+            # TODO isa_enum  - anon enum generation
+            # TODO coerce_via - anon coercion (type -> anon subtype+coercion
+
+            # XXX we also ignore conflicts here -- last in wins
+            #confess q{conflict 'isa' and 'isa_class' or 'isa_role'}
+                #if $_has->('isa')
+
+            # XXX undocumented -- not sure this is a great idea
+            $options->{isa} = class_type(delete $options->{isa_class})
+                if $_has->('isa_class');
+            $options->{isa} = role_type(delete $options->{isa_role})
+                if $_has->('isa_role');
+            $options->{isa} = enum(delete $options->{isa_enum})
+                if $_has->('isa_enum');
+
+            ### the pretty business of on-the-fly subtyping...
+            if ($_has->('constraint')) {
+
+                # check for errors...
+                $class->throw_error('You must specify an "isa" when declaring a "constraint"')
+                    if !$_has->('isa');
+                $class->throw_error('"constraint" must be a CODE reference')
+                    if $_ref->('constraint') ne 'CODE';
+
+                # constraint checking! XXX message, etc?
+                push my @opts, constraint => $_opt->('constraint')
+                    if $_ref->('constraint') eq 'CODE';
+
+                # stash our original option away and construct our new one
+                my $isa = $options->{original_isa} = $_opt->('isa');
+                my $isa_tc = _find_or_create_isa_type_constraint($isa);
+                my $new_tc = $isa_tc->create_child_type(@opts);
+
+                if ($isa_tc->has_coercion && $_opt->('coerce') eq "1") {
+
+                    # create our coercion as a copy of the parent
+                    $new_tc->coercion(Moose::Meta::TypeCoercion->new(
+                        type_constraint   => $new_tc,
+                        type_coercion_map => $isa_tc->coercion->type_coercion_map,
+                    ));
+                }
+
+                # fin constraint mucking....
+                $options->{isa} = $new_tc;
+            }
+
             if ($options->{lazy_build} && $options->{lazy_build} eq 'private') {
 
                 $options->{lazy_build} = 1;
@@ -223,6 +290,7 @@ sub init_meta {
     Moose::Util::MetaRole::apply_metaroles(
         for             => $for_class,
         class_metaroles => { attribute         => [ $role ] },
+        # TODO add attribute trait here to create builder method if found
         role_metaroles  => { applied_attribute => [ $role ] },
     );
 
@@ -237,7 +305,7 @@ __END__
 
 =encoding utf-8
 
-=for :stopwords Chris Weyl
+=for :stopwords Chris Weyl GitHub attribute's isa one's rwp
 
 =head1 NAME
 
@@ -245,7 +313,7 @@ MooseX::AttributeShortcuts - Shorthand for common attribute options
 
 =head1 VERSION
 
-This document describes version 0.016 of MooseX::AttributeShortcuts - released September 08, 2012 as part of MooseX-AttributeShortcuts.
+This document describes version 0.018 of MooseX::AttributeShortcuts - released January 09, 2013 as part of MooseX-AttributeShortcuts.
 
 =head1 SYNOPSIS
 
@@ -298,6 +366,8 @@ This package causes an attribute trait to be applied to all attributes defined
 to the using class.  This trait extends the attribute option processing to
 handle the above variations.
 
+=for Pod::Coverage init_meta
+
 =head1 USAGE
 
 This package automatically applies an attribute metaclass trait.  Unless you
@@ -438,21 +508,43 @@ e.g., in your class,
     has foo => (is => 'ro', builder => '_build_foo');
     sub _build_foo { 'bar!' }
 
-=for Pod::Coverage init_meta
+=head2 isa => ..., constraint => sub { ... }
 
-=head1 SOURCE
+Specifying the constraint option with a coderef will cause a new type
+constraint to be created, with the parent type being the type specified in the
+C<isa> option and the constraint being the coderef supplied here.
 
-The development version is on github at L<http://github.com/RsrchBoy/moosex-attributeshortcuts>
-and may be cloned from L<git://github.com/RsrchBoy/moosex-attributeshortcuts.git>
+For example, only integers greater than 10 will pass this attribute's type
+constraint:
 
-=head1 BUGS
+    # value must be an integer greater than 10 to pass the constraint
+    has thinger => (
+        isa        => 'Int',
+        constraint => sub { $_ > 10 },
+        # ...
+    );
 
-Please report any bugs or feature requests on the bugtracker website
-https://github.com/RsrchBoy/moosex-attributeshortcuts/issues
+Note that if you supply a constraint, you must also provide an C<isa>.
 
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
+=head2 isa => ..., constraint => sub { ... }, coerce => 1
+
+Supplying a constraint and asking for coercion will "Just Work", that is, any
+coercions that the C<isa> type has will still work.
+
+For example, let's say that you're using the C<File> type constraint from
+L<MooseX::Types::Path::Class>, and you want an additional constraint that the
+file must exist:
+
+    has thinger => (
+        is         => 'ro',
+        isa        => File,
+        constraint => sub { !! $_->stat },
+        coerce     => 1,
+    );
+
+C<thinger> will correctly coerce the string "/etc/passwd" to a
+C<Path::Class:File>, and will only accept the coerced result as a value if
+the file exists.
 
 =head1 AUTHOR
 
